@@ -1,14 +1,12 @@
 # -*- test-case-name: twisted.mail.test.test_smtp -*-
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Simple Mail Transfer Protocol implementation.
 """
 
-import time, re, base64, types, socket, os, random, hmac
-import MimeWriter, tempfile, rfc822
-import warnings
+import time, re, base64, types, socket, os, random, rfc822
 import binascii
 from email.base64MIME import encode as encode_base64
 
@@ -24,11 +22,8 @@ from twisted.internet import reactor
 from twisted.internet.interfaces import ITLSTransport
 from twisted.python import log
 from twisted.python import util
-from twisted.python import failure
 
 from twisted import cred
-import twisted.cred.checkers
-import twisted.cred.credentials
 from twisted.python.runtime import platform
 
 try:
@@ -1396,30 +1391,38 @@ class ESMTPClient(SMTPClient):
         self.sendLine('AUTH PLAIN ' + challenge)
 
     def esmtpState_challenge(self, code, resp):
-        auth = self._authinfo
-        del self._authinfo
-        self._authResponse(auth, resp)
+        self._authResponse(self._authinfo, resp)
 
     def _authResponse(self, auth, challenge):
         self._failresponse = self.esmtpAUTHDeclined
-
         try:
             challenge = base64.decodestring(challenge)
-        except binascii.Error, e:
+        except binascii.Error:
             # Illegal challenge, give up, then quit
             self.sendLine('*')
             self._okresponse = self.esmtpAUTHMalformedChallenge
             self._failresponse = self.esmtpAUTHMalformedChallenge
         else:
             resp = auth.challengeResponse(self.secret, challenge)
-            self._expected = [235]
-            self._okresponse = self.smtpState_from
+            self._expected = [235, 334]
+            self._okresponse = self.smtpState_maybeAuthenticated
             self.sendLine(encode_base64(resp, eol=""))
 
-        if auth.getName() == "LOGIN" and challenge == "Username:":
-            self._expected = [334]
-            self._authinfo = auth
-            self._okresponse = self.esmtpState_challenge
+
+    def smtpState_maybeAuthenticated(self, code, resp):
+        """
+        Called to handle the next message from the server after sending a
+        response to a SASL challenge.  The server response might be another
+        challenge or it might indicate authentication has succeeded.
+        """
+        if code == 235:
+            # Yes, authenticated!
+            del self._authinfo
+            self.smtpState_from(code, resp)
+        else:
+            # No, not authenticated yet.  Keep trying.
+            self._authResponse(self._authinfo, resp)
+
 
 
 class ESMTP(SMTP):
@@ -1536,7 +1539,7 @@ class ESMTP(SMTP):
         if reason.check(cred.error.UnauthorizedLogin):
             self.sendCode(535, 'Authentication failed')
         else:
-            log.err(failure, "SMTP authentication failure")
+            log.err(reason, "SMTP authentication failure")
             self.sendCode(
                 451,
                 'Requested action aborted: local error in processing')
@@ -1727,45 +1730,9 @@ class SMTPSenderFactory(protocol.ClientFactory):
         return p
 
 
-class IClientAuthentication(Interface):
-    def getName():
-        """Return an identifier associated with this authentication scheme.
 
-        @rtype: C{str}
-        """
-
-    def challengeResponse(secret, challenge):
-        """Generate a challenge response string"""
-
-
-class CramMD5ClientAuthenticator:
-    implements(IClientAuthentication)
-
-    def __init__(self, user):
-        self.user = user
-
-    def getName(self):
-        return "CRAM-MD5"
-
-    def challengeResponse(self, secret, chal):
-        response = hmac.HMAC(secret, chal).hexdigest()
-        return '%s %s' % (self.user, response)
-
-
-class LOGINAuthenticator:
-    implements(IClientAuthentication)
-
-    def __init__(self, user):
-        self.user = user
-
-    def getName(self):
-        return "LOGIN"
-
-    def challengeResponse(self, secret, chal):
-        if chal== "Username:":
-            return self.user
-        elif chal == 'Password:':
-            return secret
+from twisted.mail.imap4 import IClientAuthentication
+from twisted.mail.imap4 import CramMD5ClientAuthenticator, LOGINAuthenticator
 
 class PLAINAuthenticator:
     implements(IClientAuthentication)
@@ -1778,9 +1745,10 @@ class PLAINAuthenticator:
 
     def challengeResponse(self, secret, chal=1):
         if chal == 1:
-           return "%s\0%s\0%s" % (self.user, self.user, secret)
+            return "%s\0%s\0%s" % (self.user, self.user, secret)
         else:
-           return "%s\0%s" % (self.user, secret)
+            return "%s\0%s" % (self.user, secret)
+
 
 
 class ESMTPSender(SenderMixin, ESMTPClient):
@@ -1901,96 +1869,7 @@ def sendmail(smtphost, from_addr, to_addrs, msg, senderDomainName=None, port=25)
 
     return d
 
-def sendEmail(smtphost, fromEmail, toEmail, content, headers = None, attachments = None, multipartbody = "mixed"):
-    """Send an email, optionally with attachments.
 
-    @type smtphost: str
-    @param smtphost: hostname of SMTP server to which to connect
-
-    @type fromEmail: str
-    @param fromEmail: email address to indicate this email is from
-
-    @type toEmail: str
-    @param toEmail: email address to which to send this email
-
-    @type content: str
-    @param content: The body if this email.
-
-    @type headers: dict
-    @param headers: Dictionary of headers to include in the email
-
-    @type attachments: list of 3-tuples
-    @param attachments: Each 3-tuple should consist of the name of the
-      attachment, the mime-type of the attachment, and a string that is
-      the attachment itself.
-
-    @type multipartbody: str
-    @param multipartbody: The type of MIME multi-part body.  Generally
-      either "mixed" (as in text and images) or "alternative" (html email
-      with a fallback to text/plain).
-
-    @rtype: Deferred
-    @return: The returned Deferred has its callback or errback invoked when
-      the mail is successfully sent or when an error occurs, respectively.
-    """
-    warnings.warn("smtp.sendEmail may go away in the future.\n"
-                  "  Consider revising your code to use the email module\n"
-                  "  and smtp.sendmail.",
-                  category=DeprecationWarning, stacklevel=2)
-
-    f = tempfile.TemporaryFile()
-    writer = MimeWriter.MimeWriter(f)
-
-    writer.addheader("Mime-Version", "1.0")
-    if headers:
-        # Setup the mail headers
-        for (header, value) in headers.items():
-            writer.addheader(header, value)
-
-        headkeys = [k.lower() for k in headers.keys()]
-    else:
-        headkeys = ()
-
-    # Add required headers if not present
-    if "message-id" not in headkeys:
-        writer.addheader("Message-ID", messageid())
-    if "date" not in headkeys:
-        writer.addheader("Date", rfc822date())
-    if "from" not in headkeys and "sender" not in headkeys:
-        writer.addheader("From", fromEmail)
-    if "to" not in headkeys and "cc" not in headkeys and "bcc" not in headkeys:
-        writer.addheader("To", toEmail)
-
-    writer.startmultipartbody(multipartbody)
-
-    # message body
-    part = writer.nextpart()
-    body = part.startbody("text/plain")
-    body.write(content)
-
-    if attachments is not None:
-        # add attachments
-        for (file, mime, attachment) in attachments:
-            part = writer.nextpart()
-            if mime.startswith('text'):
-                encoding = "7bit"
-            else:
-                attachment = base64.encodestring(attachment)
-                encoding = "base64"
-            part.addheader("Content-Transfer-Encoding", encoding)
-            body = part.startbody("%s; name=%s" % (mime, file))
-            body.write(attachment)
-
-    # finish
-    writer.lastpart()
-
-    # send message
-    f.seek(0, 0)
-    d = defer.Deferred()
-    factory = SMTPSenderFactory(fromEmail, toEmail, f, d)
-    reactor.connectTCP(smtphost, 25, factory)
-
-    return d
 
 ##
 ## Yerg.  Codecs!
